package com.caimei365.tools.task;

import com.caimei365.tools.feign.CommodityFeign;
import com.caimei365.tools.mapper.BaseMapper;
import com.caimei365.tools.model.dto.message.InsideMessage;
import com.caimei365.tools.model.dto.message.MessageModel;
import com.caimei365.tools.model.dto.message.MqInfo;
import com.caimei365.tools.model.dto.message.WechatTemplateMessage;
import com.caimei365.tools.model.dto.message.enums.MessageType;
import com.caimei365.tools.model.po.ClubPo;
import com.caimei365.tools.model.po.CmUserPo;
import com.caimei365.tools.service.MessageSender;
import com.caimei365.tools.utils.SmsUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import redis.clients.jedis.commands.JedisCommands;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/12/29
 */
@Slf4j
@Configuration
@EnableScheduling
@RequiredArgsConstructor
public class StatisticsTask {
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private RedisTemplate<String, String> redisTemplate;
    @Resource
    private CommodityFeign commodityFeign;
    @Resource
    private  MessageSender messageSender;

    /**
     * 每天凌晨3点执行一次统计前一天商品访问量
     * <p>
     * cron表达式语法：秒 分 小时 日 月 周
     * 年可省略
     * * 表示所有值。
     * ? 表示不指定值。
     */
    @Scheduled(cron = "0 0 3 * * ?")
    public void countProductViews() {
        try {
            // 获取昨天日期字符串
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date());
            calendar.add(Calendar.DAY_OF_MONTH, -1);
            Date viewTime = calendar.getTime();
            String dateStr = new SimpleDateFormat("yyyyMMdd").format(viewTime);
            log.info(">>>>>>>>>>>【定时任务】统计前一天商品详情访问量，统计日期（前一天）:" + dateStr);
            // 获取Redis的Key: @Statistics(prefix = "statistics_details", field = "productId")
            String redisKey = "statistics_details:productId:" + dateStr;
            Map<String, String> result = redisTemplate.execute(
                    (RedisCallback<Map<String, String>>) connection -> (
                            (JedisCommands) connection.getNativeConnection()
                    ).hgetAll(redisKey)
            );
            if (null != result && !result.isEmpty()){
                for (Map.Entry<String, String> entry : result.entrySet()) {
                    if (null != entry && StringUtils.isNotEmpty(entry.getKey()) && StringUtils.isNotEmpty(entry.getValue())) {
                        Integer productId = Integer.parseInt(entry.getKey());
                        Integer views = Integer.parseInt(entry.getValue());
                        // 保存当前商品前一天商品详情访问量
                        baseMapper.insertProductViews(productId, views, viewTime);
                        // 更新当前商品阿里云索引
                        commodityFeign.updateProductIndex(productId);
                    }
                }
            }
            log.info(">>>>>>>>>>>【定时任务】统计前一天商品详情访问量，统计成功:" + result);
        } catch (Exception e) {
            log.error(">>>>>>>>>>>【定时任务】统计前一天商品详情访问量失败:", e);
        }
    }

    /**
     * 每天早上9点半执行一次统计前一天针对机构访问商城形式的活跃行为
     * <p>
     * cron表达式语法：秒 分 小时 日 月 周
     * 年可省略
     * * 表示所有值。
     * ? 表示不指定值。
     */
    @Scheduled(cron = "0 30 9 * * ?")
    public void recordYesterday() {
        try {
            // 获取昨天日期字符串
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date());
            String dateNewStr = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
            calendar.add(Calendar.DAY_OF_MONTH, -1);
            String dateStr = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
            log.info(">>>>>>>>>>>【定时任务】统计前一天针对机构访问商城形式的活跃行为，统计日期（前一天）:" + dateStr);
            List<Map<String, Object>> behaviorRecordYesterday = baseMapper.getBehaviorRecordYesterday(dateStr);
            for (Map<String, Object> map : behaviorRecordYesterday) {
                Integer spId = (Integer) map.get("spId");
                String name = (String) map.get("name");
                Integer count =(Integer) map.get("count");
                String pageLabel = baseMapper.getBehaviorRecordYesterdayPageLabel(spId, null, dateStr);
                String openId = baseMapper.getWechatOpenId((String) map.get("unionId"));
                //站内信 【协销前一天机构访问记录】
                MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
                insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                        .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(0))
                        .info(new InsideMessage()
                                .userType(3)
                                .messageType(3)
                                .shopTieredType(7)
                                .name(name)
                                .content(pageLabel)
                                .sum(count)
                                .thisId(spId)
                        );
                messageSender.messageSend(insideMessageMessageModel);
                //微信模板消息 【协销前一天机构访问记录】
                ArrayList<String> keyWords = new ArrayList<>();
                keyWords.add(name+"...等"+count+"家机构客户");
                keyWords.add(pageLabel);
                keyWords.add(dateStr);
                MessageModel<WechatTemplateMessage> wechatTemplateMessageMessageModel = new MessageModel<WechatTemplateMessage>()
                        .code(MessageType.WECHAT_TEMPLATE_MESSAGE)
                        .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                        .info(new WechatTemplateMessage()
                                .first("")
                                .openId(openId)
                                .appid("wxf3cd4ae0cdd11c36")
                                .templateId("Zd7ewJw9-bN-gIgra9flKW96W3YehikRPPkDXPfhmQ0")
                                .keyWords(keyWords)
                                .remark("为了更好给您服务，请及时联系采美客服!")
                                .pagePath("pages/seller/notice/service/Institutional_visits?spId="+spId+"&accDateTime="+dateNewStr));
                messageSender.messageSend(wechatTemplateMessageMessageModel);
            }
            //访问商城形式的不活跃行为 短信推送
            calendar.add(Calendar.DATE, -180);
            String dateStr2 = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
            log.info(">>>>>>>>>>>【定时任务】统计前一天针对机构访问商城形式的不活跃行为，统计日期（180天前）:" + dateStr2);
            List<ClubPo> clubList = baseMapper.getClubList(new CmUserPo().setLoginTime(dateStr2));
            for (ClubPo clubPo : clubList) {
                if (null != clubPo.getSpID()) {
                    String mobile = baseMapper.findMobile(baseMapper.findSpUserId(clubPo.getSpID()));
                    if (StringUtils.isNotBlank(mobile)) {
                        SmsUtil.sendSms(1, mobile, "【采美365】该机构客户已有6个月未在商城活跃，请及时跟进。机构名称【" + clubPo.getName() + "】，联系人【" + clubPo.getLinkMan() + "】，手机号【" + clubPo.getContractMobile() + "】。");
                    }
                }
            }
            log.info(">>>>>>>>>>>【定时任务】统计前一天针对机构访问商城形式的活跃行为，统计成功");
        } catch (Exception e) {
            log.error(">>>>>>>>>>>【定时任务】统计前一天针对机构访问商城形式的活跃行为失败:", e);
        }
    }


    /**
     * 每天五分钟执行一次统计统计内容库访问活跃行为
     * <p>
     * cron表达式语法：秒 分 小时 日 月 周
     * 年可省略
     * * 表示所有值。
     * ? 表示不指定值。
     */
    @Scheduled(cron = "0 */5 * * * ?")
    public void behaviorRecordMinute() {
        try {
            Calendar calendar = Calendar.getInstance();
            String dateNewStr = new SimpleDateFormat("yyyy-MM-dd HH:mm:00").format(calendar.getTime());
            calendar.add(Calendar.MINUTE, -5);
            String dateStr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(calendar.getTime());
            List<Map<String, Object>> behaviorRecordClubMinute = baseMapper.getBehaviorRecordClubMinute(dateStr);
            for (Map<String, Object> map : behaviorRecordClubMinute) {
                Integer spId = Integer.valueOf(map.get("spId").toString());
                String name = (String) map.get("name");
                // String openId = baseMapper.getWechatOpenId((String) map.get("unionId"));
                Integer clubID = Integer.valueOf(map.get("clubID").toString());
                //站内信
                MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
                insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                        .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                        .info(new InsideMessage()
                                .userType(3)
                                .messageType(3)
                                .shopTieredType(9)
                                .name(name)
                                .clubId(clubID)
                                .thisId(spId)
                        );
                messageSender.messageSend(insideMessageMessageModel);
            }
            List<Map<String, Object>> behaviorRecordYesterday = baseMapper.getBehaviorRecordMinute(dateStr);
            for (Map<String, Object> map : behaviorRecordYesterday) {
                Integer spId = Integer.valueOf(map.get("spId").toString());
                String name =  map.get("name").toString();
                Integer count = Integer.valueOf(map.get("count").toString());
                String openId = baseMapper.getWechatOpenId((String) map.get("unionId"));
                String pageLabel = baseMapper.getBehaviorRecordMinutePageLabel(spId, null, dateStr);
                //站内信
                MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
                insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                        .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                        .info(new InsideMessage()
                                .userType(3)
                                .messageType(3)
                                .shopTieredType(10)
                                .name(name)
                                .content(pageLabel)
                                .sum(count)
                                .thisId(spId)
                            );
                    messageSender.messageSend(insideMessageMessageModel);
                    //微信模板消息
                    ArrayList<String> keyWords = new ArrayList<>();
                    keyWords.add(name+"...等"+count+"家机构客户访问了您的内容库资料分享");
                    keyWords.add(pageLabel);
                    keyWords.add(DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd"));
                    MessageModel<WechatTemplateMessage> wechatTemplateMessageMessageModel = new MessageModel<WechatTemplateMessage>()
                            .code(MessageType.WECHAT_TEMPLATE_MESSAGE)
                            .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                            .info(new WechatTemplateMessage()
                                    .first("")
                                    .openId(openId)
                                    .appid("wxf3cd4ae0cdd11c36")
                                    .templateId("Zd7ewJw9-bN-gIgra9flKW96W3YehikRPPkDXPfhmQ0")
                                    .keyWords(keyWords)
                                    .remark("为了更好给您服务，请及时联系采美客服!")
                                    .pagePath("pages/seller/notice/service/Institutional_visits?type=1&spId="+spId+"&accDateTime="+dateNewStr));
                    messageSender.messageSend(wechatTemplateMessageMessageModel);
            }
            log.info(">>>>>>>>>>>【定时任务】统计内容库访问活跃行为，统计成功");
        } catch (Exception e) {
            log.error(">>>>>>>>>>>【定时任务】统计内容库访问活跃行为活跃行为失败:", e);
        }
    }
}
